#ifndef OBJ_H_
#define OBJ_H_

/*
 * object file
 *
 *  ------------
 * | obj_header	|
 *  ------------
 * |  section1  |
 *  ------------
 * |  section2  |
 *  ------------
 * |  		|
 *  ------------
 * |  sectionN  |
 *  ------------
 * | sect_head1 |
 *  ------------
 * | sect_head2 |
 *  ------------
 * |		|
 *  ------------
 * | sect_headN |
 *  ------------
 *
 * */

typedef enum {
	ST_UNKNWON,
	ST_GLOBAL,
	ST_MODULE
} sym_type_t;

typedef enum {
	FT_UNKNOWN,
	FT_OBJ,
	FT_EXEC
} file_type_t;

typedef enum {
	SHT_UNKNOWN,
	SHT_TEXT,
	SHT_INIT,
	SHT_DATA,
	SHT_RO,
	SHT_SYMTAB,
	SHT_RELTAB,
	SHT_STRTAB,
	SHT_UNDEF,
} sh_type_t;


#define	MAGIC_LEN	4
#define MAGIC_STRING	".VM"
#define MAJOR_VERS	0
#define MINOR_VERS	1

struct obj_header {
	unsigned char	o_magic[MAGIC_LEN];	/* ".VM" + '\0' terminator */
	file_type_t	o_type;		/* file type object or executable */
	uint16_t	o_maj_vers;
	uint16_t	o_min_vers;
	uint32_t	o_src_name;	/* source file name - file.vm */
	uint32_t	o_shoff;	/* section headers table offset in file */
	uint32_t	o_shnum;	/* number of entries in section headers table */
	uint32_t	o_shentsize;	/* the size in bytes of each entry */
	uint32_t	o_entry;	/* start execution address */
};

struct section_header {
	uint32_t	sh_name;	/* section name .data .text etc */
	sh_type_t	sh_type;	/* type, see enum above */
	uint32_t	sh_off;		/* first byte offset in file */
	uint32_t	sh_size;	/* section size */
	uint32_t	sh_entsize;	/* entry size,  for example symtable entry
					or relocation table entry */
};

struct sym_tb_entry {
	uint32_t	st_name;	/* symbol name */
	uint32_t	st_offset;	/* offset in section */
	uint32_t	st_size;	/* size of variable or function */
	sym_type_t	st_visib;	/* local, global */
	sh_type_t	st_shtype;	/* section type */
};

typedef enum {
	RF_UNKNOWN,
	RF_SYMTB,
	RF_SECT
} r_flag_t;

struct rel_tb_entry {
	uint32_t	r_info;		/* index in a symbol table or the
	 	 	 	 	 * of the section (r_flag) */
	uint32_t	r_flag;		/* how to use r_info field */
	uint32_t	r_offset;	/* offset in section */
	sh_type_t	r_shtype;	/* .init or .text */
};

struct buffer;

void obj_put_sym_tb_entry_be(struct buffer *buffer, struct sym_tb_entry *entry);
void obj_put_sym_tb_entry(struct buffer *buffer, struct sym_tb_entry *entry);
void obj_put_reloc_tb_entry_be(struct buffer *buffer, struct rel_tb_entry *entry);
void obj_put_obj_header_be(struct buffer *buffer, struct obj_header *obj);
void obj_put_section_header_be(struct buffer *buffer, struct section_header *header);

void obj_get_reloc_tb_entry(struct buffer *buffer, struct rel_tb_entry *entry);
void obj_get_sym_tb_entry(struct buffer *buffer, struct sym_tb_entry *entry);
void obj_get_section_header(struct buffer *buffer, struct section_header *header);
void obj_get_obj_header(struct buffer *buffer, struct obj_header *obj);

#endif /* OBJ_H_ */
